/* rbexp5 - modified from rbexp4b to run mixed blocks of trial types */
/* using C1 C2 X paradigm.*/

#include <stdio.h>
#include <stdlib.h>
#include <conio.h>
#include <graphics.h>
#include <string.h>
#include <ctype.h>

/* prototypes */
void inst(int);
void collect_data(char[], int);
void show_practice(FILE*, int);
void presentation(char[], int);
void instructions();
void do_block(FILE*, FILE*, int);
void start_graphics();
extern void start_clock(...);
extern void stop_clock(...);

extern int ms_counter; /* millisecond timer from MSCLOCK */

/* program constants */
#define X 300 /* screen center */
#define Y 230
#define Refresh 17
#define Warm_Up 2 /* two trials to start each block */
#define MASK_TIME Refresh /* initial mask duration, ms */
#define LETT2_TIME 160 /* initial letter presentation time, ms */
#define LETT3_TIME 160 /* initial letter presentation time, ms */
#define Field_Color 2 /* green */
#define Target_Color 4 /* red */
#define Fix_Color 7 /* light grey */
#define Blank_Color 0 /* black */
#define MSG_COLOR 7 /* light grey */
#define Num_Letters 7
#define Num_Blocks 4
#define Num_Trials 32   /* # trials in normal block */
#define Pract_Trials 14 /* # trials in practice block */
#define Num_Targ 3
#define Adjust_Rate 4 /* number of control trials per time adjustment */
#define MAX_LINE 100  /* max line length from input file */

/* Serial letter presentation program.  Built for Elwyn Leek by Scott Cost */
/* Must be compiled with MSCLOCK.CPP, RSC custom millisecond timer. */
int main (
	int argc,
	char* argv[] )
{
	int ms_lett; /* variables for letter and mask duration, in ms */
	int ms_mask;
	FILE *infile, *outfile; /* convention:  command argument 1 is in, 2 is out */
	int block;

	start_clock();
	_setcursortype(_NOCURSOR);

	if ((infile = fopen(argv[1],"r"))==NULL) {
		printf ("%s:  (In)File Open Error.\n", argv[1]);
		getch();
		exit(1);
	}
	if ((outfile = fopen(argv[2], "w"))==NULL)
	{
		printf ("%s:  (Out)File Open Error.\n", argv[2]);
		getch();
		exit(1);
	}
	inst(1);
	for (block=-2; block<Num_Blocks; block++) {
		clrscr();
		do_block(infile, outfile, block);
		clrscr();
		if (block>=0) {
			printf ("You have just completed block %d of 6.\n", block+3);
			printf ("Please wait . . .\n");
		}
		ms_counter=0;
		while (ms_counter<5000); /* pause for 5 seconds */
		/* process data */
	}
	clrscr();
	printf("This experiment is complete.  Thank you for your participation.\n");
	getch();
	fcloseall();
	stop_clock(); /* add this to an error handling routine */
	return(0); /* no error */
 }

void do_block (
	FILE *infile,
	FILE *outfile,
	int block
)
{
	static int lett2_time = LETT2_TIME;
	static int lett3_time = LETT3_TIME;
	static int id=0;
	char letter[Num_Trials][Num_Letters]; /* character array */
	char response[Num_Trials][Num_Targ+1];  /* subject responses */
	char trial_type[Num_Trials];  /* trial type identifier code */
	char line[MAX_LINE];
	char string[100]; /* presentation window */
	char a1, a2, a3, l1, l2, l3, shift, type;
	int trial, itrial, position, i, swap, dest, num_trials;
	int color[Num_Trials][Num_Letters];   /* color array */
	int lett_time, num_targ; /* current trial variables */
	int shuffle[Num_Trials];
	int one, two, thr, score_l2, score_l3, score_s2, score_s3;
	int count_c2, count_c3, count_r2, count_r3;
	int control=0, control_score=0, count;
	int ctrl_3 = 0, ctrl_2 = 0; /* hold control counts for mixed cases */
	int s_scr_2 = 0, s_scr_3 = 0;
	int ident[Num_Trials];
	float performance;
	int time; /* used as context marker - see context setting/saving */

	start_graphics();

	if (block<0) {
		num_trials=Pract_Trials;
		show_practice(infile, block);
	}
		else num_trials=Num_Trials;

	closegraph(); /* insulate text screen */

	/*******************************************/

	if (block==-2) inst(2);
	else if (block==-1) inst(4);
	else if (block==0) inst(6);
	else if (block==1) inst(7);
	else if (block==2) inst(8);
	else if (block==3) inst(9);

	/* randomization - shuffle is the index for order of presentation */
	randomize(); /* time function dependent */
	for (i=0; i<num_trials; i++) shuffle[i]=i;
	for (i=0; i<num_trials; i++) { /* swap-shuffle */
		dest=random(num_trials);
		swap=shuffle[i]; shuffle[i]=shuffle[dest]; shuffle[dest]=swap;
		}
	for (trial=0; trial<num_trials; trial++) {
		ident[trial]=id++;
		fgets(line,MAX_LINE,infile);
		position = 0;
		trial_type[trial] = (strtok(line, " \t\n"))[0];

		for (position=0; position<Num_Letters; position++) {
			letter[trial][position] = (strtok(NULL, " \t\n"))[0];
		}
	}
	start_graphics();
	/*******************************************/

	count_c2 = count_c3 = count_r2 = count_r3 = 0;

	for (trial=0; trial<num_trials; trial++) {

		itrial=shuffle[trial];

	/* The following switch statement determines what type of trial
		 we are doing and sets the parameter for that trial. Types are:
		 type code      # of targets     type         timing
		 0              2                Filler       3
		 1              2                Control      2
		 2              2                Repeat       2
		 3              2                Filler       2
		 4              3                Control      3    C1 Y C2
		 5              3                Repeat       3
		 6              3                Filler       3
		 7              3                Control      2    C1 C2 Y
		 8              3                Repeat       2
		 9              3                Filler       2
	*/
	switch (trial_type[itrial]) {
	       case '0': /* Filler 1 */
			 num_targ=2; type='F'; lett_time=lett3_time; time=3; break;
	       case '1': /* Control 2 */
			 num_targ=2; type='C'; lett_time=lett2_time; time=2; break;
	       case '2': /* Repeat 2 */
			 num_targ=2; type='R'; lett_time=lett2_time; time=2; break;
	       case '3': /* Filler 2 */
			 num_targ=2; type='F'; lett_time=lett2_time; time=2; break;
	       case '4': /* Control 3 - Time 3 */
			 num_targ=3; type='C'; lett_time=lett3_time; time=3; break;
	       case '5': /* Repeat 3 */
			 num_targ=3; type='R'; lett_time=lett3_time; time=3; break;
	       case '6': /* Filler 3*/
			 num_targ=3; type='F'; lett_time=lett3_time; time=3; break;
	       case '7': /* Control 3 - Time 2 */
			 num_targ=2; type='C'; lett_time=lett2_time; time=2; break;
	       case '8': /* Repeat 3 */
			 num_targ=2; type='R'; lett_time=lett2_time; time=2; break;
	       case '9': /* Filler 3 */
			 num_targ=2; type='F'; lett_time=lett3_time; time=3; break;
	       default:  
			 printf("\n%c%c%cBad trial-type identifier code. Program dies.\n",7,7,7);
			 stop_clock(); fcloseall(); closegraph(); exit(1); break;
	}                          
	/***** end block mixture setting *****/

		presentation(letter[itrial], lett_time);
		collect_data(response[itrial], block);


		fprintf (outfile, "%2d %d %d %c %d", block, trial, itrial, type, time);
		fprintf (outfile, " <%c %c %c %c %c %c %c>", letter[itrial][0],
			letter[itrial][1], letter[itrial][2], letter[itrial][3],
			letter[itrial][4], letter[itrial][5], letter[itrial][6] );
		fprintf (outfile, " <%c %c %c>", response[itrial][0],
			response[itrial][1], response[itrial][2] );
		one=two=thr=0; /* place markers for key positions */
		if (type!='F') {one=2; two=3; thr=4;}
		else if (isupper(letter[itrial][0])) {one=0; two=1; thr=2;}
		else {one=4; two=5; thr=6;}

		/* letters l1 - l3 correct, a1 and a3 always contain outside targets */
		/* for 2 or 3 target case */


		l1=toupper(letter[itrial][one]); l2=toupper(letter[itrial][two]);
		l3=toupper(letter[itrial][thr]); a1=response[itrial][0];
		a2=response[itrial][1]; a3=response[itrial][2];
		if (/*num_targ*/3==2) {
			shift=a3; a3=a2; a2=shift; /* shift response */ a2='*';}
		score_l2=score_s2=score_l3=score_s3=9;
		if (((l1==a1) & (l3==a3)) | ((l1==a3) & (l3==a1))) score_l2=2;
		else if ((l1==a1) | (l1==a3) | (l3==a1) | (l3==a3)) score_l2=1;
		else score_l2=0;
		if ((l1==a1) & (l3==a3)) score_s2=2;
		else if ((l1==a1) | (l3==a3)) score_s2=1;
		else score_s2=0;
		if (((l1==a1) & (l3==a2)) |
				((l1==a1) & (l2==a3)) |
				((l1==a1) & (l3==a3)) |
				((l1==a2) & (l3==a1)) |
				((l1==a2) & (l3==a3)) |
				((l1==a3) & (l3==a1)) |
				((l1==a3) & (l3==a2))) score_l3=2;
		else if ((a1==l1)|(a1==l3)|(a2==l1)|(a2==l3)|(a3==l1)|(a3==l3))
			score_l3=1;
		else score_l3=0;
		if ((l1==a1) & (l2==a2) & (l3==a3)) score_s3=3;
		else if (score_s2==2) score_s3=2;
		else if ((score_s2==1) & (l2==a2)) score_s3=2;
		else if ((score_s2==1) | (l2==a2)) score_s3=1;
		else score_s3=0;

		fprintf (outfile, "[%d] l2:%d l3:%d s2:%d s3:%d", ident[trial], score_l2, score_l3, score_s2, score_s3);
		fprintf (outfile, " %d\n", lett_time); /* prior to rescoring */

		if (trial==(num_trials-1))
			fprintf (outfile, "c2: %d c3: %d r2: %d r3: %d\n",
			count_c2, count_c3, count_r2, count_r3);

		/* unpackage score cards for this trial type */
		if (time==2) {control=ctrl_2; control_score=s_scr_2;}
		else {control=ctrl_3; control_score=s_scr_3;}

		/* tabulate - NOT used for context-dependent timing */
		if (score_l3==2) {
			if (type=='C') {
				if (num_targ==2) count_c2++;
				else if (num_targ==3) count_c3++;
			}
			else if (type=='R') {
				if (num_targ==2) count_r2++;
				else if (num_targ==3) count_r3++;
			}
		}

		if (type=='C') {
			control++; /* one more control trial */
			if (score_l3==2) control_score++; /* no partial scoring */
		}
		if (control==Adjust_Rate) { /* time to step the rate */
			if (control_score==4) lett_time-=(Refresh*2);
			else if (control_score==3) lett_time-=Refresh;
			else if (control_score==1) lett_time+=Refresh;
			else if (control_score==0) lett_time+=(Refresh*2);

			control_score=control=0;
		} /* end scoring module */

		/* context save */ /* important - see context setting above */
		if (time==2)
			{lett2_time=lett_time; ctrl_2=control; s_scr_2=control_score;}
		else if (time==3)
			{lett3_time=lett_time; ctrl_3=control; s_scr_3=control_score;}

		setcolor (3);
		if (itrial==(num_trials+1))
			strcpy (string, "Block completed.  Hit RETURN to continue:");
		else strcpy (string, "Hit RETURN to begin next trial:");
		outtextxy(10, 10, string);
		while (getch()!=13);
		cleardevice();
	}
	/* store data to file */
	closegraph();
	if (block==-2) inst(3);
	else if (block==-1) inst(5);
	/* end block */
}

void show_practice(
	FILE *infile,
	int block
)
{

	char letter[2][Num_Letters]; /* character array */
	char response[2][Num_Targ+1];  /* subject responses */
	int trial, position, i;
	char line[MAX_LINE];
	char string[100]; /* presentation window */
	int lett_time; /* current trial variables */

	for (trial=0; trial<2; trial++) {
		fgets(line,MAX_LINE,infile);
		position = 0;
		letter[trial][position] = (strtok(line, " \t\n"))[0];
		for (position=1; position<Num_Letters; position++)
			letter[trial][position] = (strtok(NULL, " \t\n"))[0];
	}

	setcolor(MSG_COLOR);
	strcpy(string, "You will now be given two example trials.  Hit RETURN:\n");
	outtextxy(10,40,string);
	while (getch()!=13);

	cleardevice();

	for (i=0; i<2; i++) {
		presentation(letter[i], LETT2_TIME);
		collect_data(response[i], block);

		strcpy (string, "Hit RETURN to continue:");
		outtextxy(10, 10, string);
		while (getch()!=13);

		cleardevice();
	}
	cleardevice();
}

/* vga graphics initialization routine */
void start_graphics() {
	int gdriver=VGA, gmode=VGAHI, errorcode;
	initgraph(&gdriver, &gmode, "C:\\BORLANDC\\BGI\\"); /* enter graphics mode */
	errorcode=graphresult();
	if (errorcode!=grOk) {
		printf("Graphics error:  %s\n", grapherrormsg(errorcode));
		printf("Press any key to halt:");
		getch();
		exit(1);
	} /* from BC++ Lib Ref, pg. 299 */
	settextstyle(TRIPLEX_FONT,HORIZ_DIR,1);
	/* setusercharsize(); for future use; font size must be set to zero */
}

/* an error handler */

/* present a string of letters in serial fashion */
void presentation(
	char letter[],
	int lett_time
	)
	{
		int position;
		char string[2];

		/* fixation point */
		string[0]=' '; string[1]='\0'; /* set up character frame */
		ms_counter=0; while (ms_counter<1000);
		setcolor(Fix_Color); string[0]='+'; outtextxy(X, Y, string);
		ms_counter=0; while (ms_counter<1000);
		setcolor(Blank_Color); outtextxy (X, Y, string);
		ms_counter=0; while (ms_counter<500);

		for (position=0; position<Num_Letters; position++) {
			if (islower(letter[position])) {
				string[0]=toupper(letter[position]); setcolor(Field_Color); }
			else { string[0]=letter[position]; setcolor(Target_Color); }
			if (isdigit(letter[position])) setcolor(Target_Color);
			ms_counter=0; /* RESET TIMER */
			outtextxy(X, Y, string);  /* display character */
			/* wait for next mark */
			while (ms_counter<lett_time); ms_counter=0;  /* TIME PRESENTATION */
			setcolor (Blank_Color); /* background - mask */
			outtextxy(X, Y, string);
			while (ms_counter<MASK_TIME); /* TIME MASK */
		}
}

void collect_data( /* DATA COLLECTION ROUTINE */
	char response[],
	int block
)
{
	char string[100];
	int i;
	int num_targ=3;
	strcpy (string, "Please enter the target characters in order, then RETURN:");
	setcolor (MSG_COLOR);
	ms_counter=0;  while (ms_counter<1000);
	outtextxy(10, 10, string);

	response[1]='*';
	response[2]='*';
	response[3]='*'; /* default */
	for (i=0; i<num_targ+1; i++) {
		response[i]=toupper(getch());
		/* escape hatch - hit '@' as a response */
		if (response[i]=='@') {stop_clock(); fcloseall(); closegraph(); exit(1);}
		if (response[i]==13) {response[i]='*'; break;}
		if ((isalpha(response[i])==0)&(isdigit(response[i])==0)) response[i]='*';
	}
	if ((response[0]=='0')&(response[1]=='0')) {stop_clock(); fcloseall();
																							closegraph(); exit(1);}
	cleardevice();
}

void inst (
	int mssg
) {
	/* do good things, then */
	clrscr();
	if (mssg==1) {
	/* Text for 1st screen: */

	printf ("Thank you for participating in this study. The experiment consists\n");
	printf ("of a number of trials. Each trial contains several colored letters.\n");
	printf ("Some are green, others are red. All letters are consonants. On each\n");
	printf ("trial, you will see either 2 or 3 red letters. Your task is to\n");
	printf ("report the identity of each red letter after the trial is finished.\n");
	printf ("The trials are organized into 4 blocks. In each block most of the trials\n");
	printf ("will have 3 target letters but there will always be some with only 2 red\n");
	printf ("letters. It is very important that you try to correctly identify as\n");
	printf ("many of the target letters in each trial as you can. Report the identity\n");
	printf ("of the red letter targets after each trial by typing the letters you saw\n");
	printf ("into the keyboard in front of you, in the order they appeared.\n");

	printf ("You will now be shown two demonstration trials.\n");
	}

	else if (mssg==2) {
	/* Text for screen 2 (after demo block 1): */
	printf ("This is the first practice block.  After each trial, report the\n");
	printf ("identity of all RED letters in the order they were presented.\n");
	printf ("Please press return when you are ready to go on.\n");
	}

	else if (mssg==3) {
	/* Text for screen 3 (after practice block 1) */
	printf ("This block of practice trials is now finished. You will now see\n");
	printf ("two more demonstration trials.\n");
	}

	else if (mssg==4) {
	/* Text for screen 4 (after 3 letter demonstration trials) */
	printf ("You will now be given the second practice block. Please press RETURN\n");
	printf ("when you are ready to begin. The purpose of this block is to see\n");
	printf ("how fast the letters should be presented to you. Don't be alarmed if\n");
	printf ("the speed changes between trials!\n");
	}

	else if (mssg==5) {
	/* Text for screen 5 (after practice block 2) */
	printf ("The practice trials are finished. Please read carefully the following\n");
	printf ("additional instructions before completing the rest of the trials.\n");
	printf ("You will now be shown four more blocks of trials. In each block most\n");
	printf ("of the trials will have 3 red target letters but there will always\n");
	printf ("be some trials with only 2 red letters. On some of the trials the same\n");
	printf ("letter may be repeated\n");

	printf ("You must report all red letters including repeated ones.\n");

	printf ("Your responses are NOT timed. If you get tired, please feel free\n");
	printf ("to rest after each block. If you have any questions, please ask the\n");
	printf ("experimenter now.\n");

	printf ("\nGOOD LUCK !\n");
	}

	else if (mssg==6) {
	printf (" ** BLOCK 1 ** \n");
	}

	else if (mssg==7) {
	printf (" ** BLOCK 2 ** \n");
	}

	else if (mssg==8) {
	printf (" ** BLOCK 3 ** \n");
	}

	else if (mssg==9) {
	printf (" ** BLOCK 4 ** \n");
	}

	printf ("\nHit RETURN to continue . . .\n");
	while (getch()!=13);
	clrscr();
}
